"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated

from encommon.types import BaseModel

from pydantic import Field

from ...robie.params import RobiePluginParams



class StatusPluginCommandParams(BaseModel, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    irc: Annotated[
        str,
        Field('!status',
              description='Command name for chat platform')]

    dsc: Annotated[
        str,
        Field('!status',
              description='Command name for chat platform')]

    mtm: Annotated[
        str,
        Field('!status',
              description='Command name for chat platform')]



class StatusPluginParams(RobiePluginParams, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    command: Annotated[
        StatusPluginCommandParams,
        Field(default_factory=StatusPluginCommandParams,
              description='Command name per chat platform')]



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

from enconnect.mattermost import ClientEvent

from pytest import raises

from ..command import MTMCommand
from ..message import MTMMessage

if TYPE_CHECKING:
    from ....robie import Robie



EVENT = {
    'event': 'posted',
    'seq': 5,
    'broadcast': {
        'channel_id': 'nwyxekd4k7'},
    'data': {
        'channel_type': 'P',
        'post': (
            '{"user_id":"ietyrmdt5b",'
            '"channel_id":"nwyxekd4k7",'
            '"message":"Hello"}'),
        'sender_name': '@robert'}}



def test_MTMMessage(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    childs = robie.childs
    clients = childs.clients

    model = MTMMessage

    event = ClientEvent(EVENT)

    client = clients['mtmbot']


    item = model(
        client, event)


    attrs = lattrs(item)

    assert attrs == [
        'event',
        'client',
        'time']


    assert inrepr(
        'MTMMessage',
        item)

    with raises(TypeError):
        assert hash(item) > 0

    assert instr(
        'MTMMessage',
        item)


    assert item.time.since > 0

    assert item.client == client.name

    assert item.family == 'mattermost'

    assert item.kind == 'chanmsg'

    assert item.event == event


    assert event.type == 'posted'
    assert event.data
    assert len(event.data) == 3
    assert event.broadcast
    assert len(event.broadcast) == 1
    assert event.seqno == 5
    assert not event.status
    assert not event.error
    assert not event.seqre

    assert event.kind == 'chanmsg'
    assert event.author == (
        'ietyrmdt5b', '@robert')
    assert event.recipient == 'nwyxekd4k7'
    assert event.message == 'Hello'



def test_MTMMessage_reply(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    childs = robie.childs
    clients = childs.clients

    model = MTMMessage


    item = model(
        clients['mtmbot'],
        ClientEvent(EVENT))

    reply = item.reply(
        robie, 'Hello')


    assert isinstance(
        reply, MTMCommand)


    assert reply.family == 'mattermost'

    assert reply.method == 'post'

    assert reply.path == 'posts'

    assert not reply.params

    assert reply.json == {
        'channel_id': 'nwyxekd4k7',
        'message': 'Hello'}



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from contextlib import suppress
from typing import Optional
from typing import Type

from encommon.config import Config
from encommon.config import Params
from encommon.types import DictStrAny
from encommon.utils.common import PATHABLE

from .params import RobieClientParams
from .params import RobieParams
from .params import RobiePluginParams



class RobieConfig(Config):
    """
    Contain the configurations from the arguments and files.

    :param sargs: Additional arguments on the command line.
    :param files: Complete or relative path to config files.
    :param cargs: Configuration arguments in dictionary form,
        which will override contents from the config files.
    """

    __clients: dict[str, Type[RobieClientParams]]
    __plugins: dict[str, Type[RobiePluginParams]]


    def __init__(
        self,
        sargs: Optional[DictStrAny] = None,
        files: Optional[PATHABLE] = None,
        cargs: Optional[DictStrAny] = None,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        sargs = dict(sargs or {})
        cargs = dict(cargs or {})

        self.__clients = {}
        self.__plugins = {}


        _console = (
            sargs.get('console'))

        _debug = (
            sargs.get('debug'))

        key = 'enlogger/stdo_level'

        if _console is True:
            cargs[key] = 'info'

        if _debug is True:
            cargs[key] = 'debug'


        if 'config' in sargs:
            files = sargs['config']


        _pmessage = (
            sargs.get('pmessage'))

        _pcommand = (
            sargs.get('pcommand'))

        if _pmessage is not None:
            key = 'printer/message'
            cargs[key] = _pmessage

        if _pcommand is not None:
            key = 'printer/command'
            cargs[key] = _pcommand


        super().__init__(
            files=files,
            cargs=cargs,
            sargs=sargs,
            model=RobieParams)


    @property
    def params(
        self,
    ) -> RobieParams:
        """
        Return the Pydantic model containing the configuration.

        .. warning::
           This method completely overrides the parent but is
           based on that code, would be unfortunate if upstream
           changes meant this breaks or breaks something else.

        :returns: Pydantic model containing the configuration.
        """

        params = self.__params

        if params is not None:

            assert isinstance(
                params, RobieParams)

            return params


        basic = self.basic
        update = False

        with suppress(AttributeError):
            basic = self.merge
            update = True


        _clients = basic.get('clients')
        _plugins = basic.get('plugins')


        def _put_clients() -> None:

            models = self.__clients
            target = basic['clients']

            items = target.items()

            for name, params in items:

                model = models[name]
                object = model(**params)

                target[name] = object


        if _clients is not None:
            _put_clients()


        def _put_plugins() -> None:

            models = self.__plugins
            target = basic['plugins']

            items = target.items()

            for name, params in items:

                model = models[name]
                object = model(**params)

                target[name] = object


        if _plugins is not None:
            _put_plugins()


        params = (
            self.model(**basic))

        assert isinstance(
            params, RobieParams)


        if update is True:
            self.__params = params

        return params


    @property
    def __params(
        self,
    ) -> Optional[Params]:
        """
        Return the Pydantic model containing the configuration.

        :returns: Pydantic model containing the configuration.
        """

        return self._Config__params


    @__params.setter
    def __params(
        self,
        value: Params,
    ) -> None:
        """
        Update the value for the attribute from class instance.
        """

        self._Config__params = value


    @property
    def config(
        self,
    ) -> DictStrAny:
        """
        Return the configuration dumped from the Pydantic model.

        .. warning::
           This method completely overrides the parent but is
           based on that code, would be unfortunate if upstream
           changes meant this breaks or breaks something else.

        :returns: Configuration dumped from the Pydantic model.
        """

        params = self.params

        _clients = params.clients
        _plugins = params.plugins

        config = params.endumped


        def _get_clients() -> None:

            target = config['clients']

            assert _clients is not None

            items = _clients.items()

            for name, client in items:
                target[name] = (
                    client.endumped)


        if _clients is not None:
            _get_clients()


        def _get_plugins() -> None:

            target = config['plugins']

            assert _plugins is not None

            items = _plugins.items()

            for name, plugin in items:
                target[name] = (
                    plugin.endumped)


        if _plugins is not None:
            _get_plugins()


        return config


    def register(
        self,
        name: str,
        *,
        client: Optional[Type[RobieClientParams]] = None,
        plugin: Optional[Type[RobiePluginParams]] = None,
    ) -> None:
        """
        Register the plugin parameters for parameter processing.

        :param name: Name of the object within the Robie config.
        :param client: Class definition for the instantiation.
        :param plugin: Class definition for the instantiation.
        """

        clients = self.__clients
        plugins = self.__plugins

        if client is not None:
            clients[name] = client

        if plugin is not None:
            plugins[name] = plugin



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated

from enconnect.irc import ClientParams

from pydantic import Field

from ...robie.params import RobieClientParams



class IRCClientParams(RobieClientParams, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    client: Annotated[
        ClientParams,
        Field(...,
              description='Parameters for the base client')]



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

from enconnect.irc import ClientEvent

from pytest import raises

from ..command import IRCCommand
from ..message import IRCMessage

if TYPE_CHECKING:
    from ....robie import Robie



EVENT = (
    ':n!u@h PRIVMSG #chan'
    ' :Hello channel')



def test_IRCMessage(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    childs = robie.childs
    clients = childs.clients

    model = IRCMessage

    event = ClientEvent(EVENT)

    client = clients['ircbot']


    item = model(
        client, event)


    attrs = lattrs(item)

    assert attrs == [
        'event',
        'client',
        'time']


    assert inrepr(
        'IRCMessage',
        item)

    with raises(TypeError):
        assert hash(item) > 0

    assert instr(
        'IRCMessage',
        item)


    assert item.time.since > 0

    assert item.client == client.name

    assert item.family == 'irc'

    assert item.kind == 'chanmsg'

    assert item.event == event


    assert event.prefix == 'n!u@h'
    assert event.command == 'PRIVMSG'
    assert event.params
    assert len(event.params) == 20
    assert len(event.original) == 35

    assert event.kind == 'chanmsg'
    assert event.author == 'n'
    assert event.recipient == '#chan'
    assert event.message == (
        'Hello channel')



def test_IRCMessage_reply(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    childs = robie.childs
    clients = childs.clients

    model = IRCMessage


    item = model(
        clients['ircbot'],
        ClientEvent(EVENT))

    reply = item.reply(
        robie, 'Hello')


    event = ClientEvent(
        EVENT.replace(
            '#chan', 'ircbot'))

    client = clients['ircbot']

    item = model(
        client, event)

    reply = item.reply(
        robie, 'Hello')


    assert isinstance(
        reply, IRCCommand)


    assert reply.event == (
        'PRIVMSG n :Hello')



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .status import StatusPlugin
from .status import StatusPluginParams



__all__ = [
    'StatusPlugin',
    'StatusPluginParams']



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .child import RobieChildParams



class RobieClientParams(RobieChildParams, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated
from typing import Optional

from encommon.config import Params
from encommon.types import BaseModel

from pydantic import Field

from .client import RobieClientParams
from .plugin import RobiePluginParams
from .service import RobieServiceParams



class RobiePrinterParams(BaseModel, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    message: Annotated[
        bool,
        Field(False,
              description='Print the messages to console')]

    command: Annotated[
        bool,
        Field(False,
              description='Print the commands to console')]



class RobieParams(Params, extra='forbid'):
    """
    Process and validate the core configuration parameters.
    """

    database: Annotated[
        str,
        Field('sqlite:///:memory:',
              description='Database connection string',
              min_length=1)]

    printer: Annotated[
        RobiePrinterParams,
        Field(default_factory=RobiePrinterParams,
              description='Print the stream to console')]

    service: Annotated[
        RobieServiceParams,
        Field(default_factory=RobieServiceParams,
              description='Parameters for Robie Service')]

    clients: Annotated[
        Optional[dict[str, RobieClientParams]],
        Field(None,
              description='Parameters for Robie clients',
              min_length=1)]

    plugins: Annotated[
        Optional[dict[str, RobiePluginParams]],
        Field(None,
              description='Parameters for Robie plugins',
              min_length=1)]



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated

from encommon.types import BaseModel

from pydantic import Field



class RobieChildParams(BaseModel, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    enable: Annotated[
        bool,
        Field(False,
              description='Determine whether plugin enabled')]
